package cn.iocoder.yudao.module.wealth.service.receivable;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivable.ReceivableDO;
import cn.iocoder.yudao.module.wealth.vo.receipt.ReceiptExportDto;
import cn.iocoder.yudao.module.wealth.vo.receivable.*;
import org.checkerframework.checker.units.qual.C;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 应收款 Service 接口
 *
 * @author 郑屹
 */
public interface ReceivableService extends IService<ReceivableDO> {

    /**
     * 创建应收款--外部接口和内部通用接口
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createReceivable(@Valid ReceivableCreateReqVO createReqVO);

    /**
     * 根据应收ID,取消应收--外部接口会校验应收不能取消
     *
     * @param id
     */
    void cancelReceivable(Long id);

    /**
     * 更新应收款--外部接口，会判断应收能不能进行修改
     *
     * @param updateReqVO 更新信息
     */
    void updateReceivableById(ReceivableDO updateReqVO);

    /**
     * 校验订单是否有对应的应收单已核销或核销中
     *
     * @param orderId     订单ID  有订单id则以订单id为主
     * @param orderItemId 订单项ID 没有订单id必须有订单项id
     */
    void verificationReceivableIsWriteOff(Long orderId, Long orderItemId);

    /**
     * 更新应收款--内部接口，不会判断应收能不能进行修改，判断前置
     *
     * @param updateReqVO 更新信息
     */
    void updateReceivable(@Valid ReceivableUpdateReqVO updateReqVO);

    /**
     * 删除应收款--内部接口,不会判断应收能不能进行删除，判断前置
     *
     * @param id 编号
     */
    void deleteReceivable(Long id);

    /**
     * 删除应收款--内部接口,不会判断应收能不能进行删除，判断前置
     *
     * @param orderId 编号
     */
    void deleteReceivableByOrderId(Long orderId);

    /**
     * 删除应收款--内部接口,不会判断应收能不能进行删除，判断前置
     *
     * @param orderId       编号
     * @param feeSourceList 费用来源 1 订单计算 2 费用申请 3 特性费用
     */
    void deleteReceivableByOrderIdAndFeeSource(Long orderId, Collection<Integer> feeSourceList);

    /**
     * 删除应收款--内部接口,不会判断应收能不能进行删除，判断前置
     *
     * @param orderId   订单ID
     * @param feeSource 费用来源：1 订单计算 2 费用申请 3 特需费用
     */
    void deleteReceivableByOrderIdAndFeeSource(Long orderId, Integer feeSource);

    /**
     * 删除应收款--内部接口,不会判断应收能不能进行删除，判断前置必须校验该订单项应收状态为待核销
     *
     * @param orderItemId 编号
     */
    void deleteReceivableByOrderItemId(Long orderItemId);


    /**
     * 删除应收款--内部接口,不会判断应收能不能进行删除，判断前置必须校验该订单项应收状态为待核销
     *
     * @param orderItemId 编号
     */
    ReceivableDO getReceivableByOrderItemId(Long orderItemId, Integer feeType);

    /**
     * 获得应收款
     *
     * @param id 编号
     * @return 应收款
     */
    ReceivableDO getReceivable(Long id);

    /**
     * 获得应收款列表
     *
     * @param ids 编号
     * @return 应收款列表
     */
    List<ReceivableDO> getReceivableList(Collection<Long> ids);

    /**
     * 获得应收款分页
     *
     * @param page  分页查询
     * @param query 查询
     * @return 应收款分页
     */
    PageResult<ReceivableDO> getReceivablePage(ReceivableQueryVO query, PageVO page);

    /**
     * 获得应收款分页
     *
     * @param page  分页查询
     * @param query 查询
     * @return 应收款分页
     */
    PageResult<ReceivableBackVO> receivablePage(ReceivableQueryVO query, PageVO page);

    /**
     * 获得应收款分页
     *
     * @param orderId 订单ID
     * @return 应收款分页
     */
    List<ReceivableBackVO> getReceivableByOrderId(Long orderId);

    /**
     * 获得应收款列表, 用于 Excel 导出
     *
     * @param query 查询
     * @return 应收款列表
     */
    List<ReceivableDO> getReceivableList(ReceivableQueryVO query);

    /**
     * 创建收款单
     *
     * @param receivableId 应收ID
     * @return 收款单ID
     */
    Long createReceipt(Long receivableId);

    List<ReceivableBackVO> receivableList(List<Long> ids);

    /**
     * 根据付款单ID得到付款单明细列表
     *
     * @param id
     * @return
     */
    List<ReceivableBackVO> getReceivableListByReceiptId(Long id);

    /**
     * 根据应付款ID得到应付详情
     *
     * @param id
     * @return
     */
    ReceivableBackVO getReceivableBackVo(Long id);


    /**
     * 根据订单ID获得应收款列表
     *
     * @param orderId
     * @return
     */
    List<ReceivableBackVO> getReceivableListByOrderId(Long orderId);

    /**
     * 根据订单ID获得需要预付应收款列表
     *
     * @param orderId
     * @return
     */
    List<ReceivableBackVO> getFirstReceivableListByOrderId(Long orderId);

    /**
     * 根据应收款ID修改优惠信息
     *
     * @param receivableUpdateReqVO
     * @return
     */
    void updateReceivableDiscountById(ReceivableUpdateReqVO receivableUpdateReqVO);

    /**
     * 根据订单id获得订单id每个应收明细的付款状态
     *
     * @param orderId
     * @return
     */
    List<ReceivableOrderPayedFeeTypeVO> getPayedMoneyByFeeType(Long orderId);

    /**
     * 添加订单应收到收款单
     * 注：创建完收款单后执行
     *
     * @param orderIdList 订单id集合
     * @param receiptNo   付款单号
     */
    void orderReceivableAddReceiptLog(Collection<Long> orderIdList, String receiptNo);

    /**
     * 添加订单应收核销日志
     * 注：需要执行完核销业务，更改应收状态为核销后才调用此方法进行日志添加判断
     *
     * @param orderIdList 订单id集合
     * @param receiptNo   付款单号
     */
    void orderReceivableWriteOffLogAdd(Collection<Long> orderIdList, String receiptNo);


    /**
     * 添加订单应收反核销日志
     *
     * @param orderIdList 订单id集合
     * @param receiptNo   付款单号
     */
    void orderReceivableCancelWriteOffLogAdd(Collection<Long> orderIdList, String receiptNo);


    List<ReceivableDO> receivableListByOrderId(Long orderId);

    List<ReceivableDO> allByOrderId(Long orderId);

    List<ReceivableDO> getOrderPayList(Collection<Long> orderIdList);

    /**
     * 根据应收id撤销删除的应收单
     *
     * @param receivableId 应收id
     */
    void revokeDeleteReceivableById(Long receivableId);
}
